@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP                #-}
 {-# LANGUAGE DerivingStrategies #-}
 {-# LANGUAGE OverloadedStrings  #-}
 {-# LANGUAGE RecordWildCards    #-}
@@ -24,7 +25,7 @@ import           Data.Function                            ((&))
 import           Data.Functor                             ((<&>))
 import           Data.Functor.Identity
 import           Data.List.NonEmpty                       (NonEmpty ((:|)))
-import qualified Data.List.NonEmpty                       as NE
+import qualified Data.List.NonEmpty                       as NE hiding (unzip)
 import           Data.Map.Strict                          (Map)
 import qualified Data.Map.Strict                          as Map
 import qualified Data.Text                                as T
@@ -40,6 +41,12 @@ import           Text.Megaparsec.Char                     (alphaNumChar, char,
                                                            eol, hspace,
                                                            letterChar)
 
+#if MIN_VERSION_base(4,19,0)
+import qualified Data.Functor                             as NE (unzip)
+#else
+import qualified Data.List.NonEmpty                       as NE (unzip)
+#endif
+
 {-
 We build parsers combining the following three kinds of them:
 
@@ -57,7 +64,7 @@ We build parsers combining the following three kinds of them:
 -}
 
 -- | Line parser
-type LineParser a = forall m. Monad m => ParsecT Void String m a
+type LineParser a = forall m. ParsecT Void String m a
 
 -- | Line comment group parser
 type LineGroupParser = Parsec Void [(Range, RawLineComment)]
@@ -122,7 +129,7 @@ commentsToSections isLHS Comments {..} =
                      in case parseMaybe lineGroupP $ NE.toList lcs of
                             Nothing -> mempty
                             Just (mls, rs) ->
-                                ( maybe mempty (uncurry Map.singleton) ((theRan,) <$> mls)
+                                ( maybe mempty (Map.singleton theRan) mls
                                 , -- orders setup sections in ascending order
                                   if null rs
                                     then mempty
