@@ -5,69 +5,67 @@
 {-# LANGUAGE LambdaCase            #-}
 {-# LANGUAGE MagicHash             #-}
 {-# LANGUAGE OverloadedStrings     #-}
+{-# LANGUAGE PatternSynonyms       #-}
 {-# LANGUAGE RecordWildCards       #-}
 {-# LANGUAGE TypeFamilies          #-}
 {-# LANGUAGE ViewPatterns          #-}
-{-# LANGUAGE PatternSynonyms       #-}
 
-module Ide.Plugin.Splice
-    ( descriptor,
-    )
-where
+module Ide.Plugin.Splice (descriptor) where
 
-import           Control.Applicative             (Alternative ((<|>)))
-import           Control.Arrow                   ( Arrow(first) )
-import           Control.Exception              ( SomeException )
-import qualified Control.Foldl                   as L
-import           Control.Lens                    (Identity (..), ix, view, (%~),
-                                                  (<&>), (^.))
-import           Control.Monad                   ( guard, unless, forM )
-import           Control.Monad.Error.Class       ( MonadError(throwError) )
-import           Control.Monad.Extra             (eitherM)
-import qualified Control.Monad.Fail              as Fail
-import           Control.Monad.IO.Unlift         ( MonadIO(..), askRunInIO )
-import           Control.Monad.Trans.Class       ( MonadTrans(lift) )
-import           Control.Monad.Trans.Except      ( ExceptT(..), runExceptT )
+import           Control.Applicative                   (Alternative ((<|>)))
+import           Control.Arrow                         (Arrow (first))
+import           Control.Exception                     (SomeException)
+import qualified Control.Foldl                         as L
+import           Control.Lens                          (Identity (..), ix, view,
+                                                        (%~), (<&>), (^.))
+import           Control.Monad                         (forM, guard, unless)
+import           Control.Monad.Error.Class             (MonadError (throwError))
+import           Control.Monad.Extra                   (eitherM)
+import qualified Control.Monad.Fail                    as Fail
+import           Control.Monad.IO.Unlift               (MonadIO (..),
+                                                        askRunInIO)
+import           Control.Monad.Trans.Class             (MonadTrans (lift))
+import           Control.Monad.Trans.Except            (ExceptT (..),
+                                                        runExceptT)
 import           Control.Monad.Trans.Maybe
-import           Data.Aeson                      hiding (Null)
-import qualified Data.Bifunctor                  as B (first)
-import           Data.Foldable                   (Foldable (foldl'))
+import           Data.Aeson                            hiding (Null)
+import qualified Data.Bifunctor                        as B (first)
 import           Data.Function
 import           Data.Generics
-import qualified Data.Kind                       as Kinds
-import           Data.List                       (sortOn)
-import           Data.Maybe                      (fromMaybe, listToMaybe,
-                                                  mapMaybe)
-import qualified Data.Text                       as T
+import qualified Data.Kind                             as Kinds
+import           Data.List                             (sortOn)
+import           Data.Maybe                            (fromMaybe, listToMaybe,
+                                                        mapMaybe)
+import qualified Data.Text                             as T
 import           Development.IDE
 import           Development.IDE.Core.PluginUtils
-import           Development.IDE.GHC.Compat      as Compat hiding (getLoc)
+import           Development.IDE.GHC.Compat            as Compat
 import           Development.IDE.GHC.Compat.ExactPrint
-import qualified Development.IDE.GHC.Compat.Util as Util
+import qualified Development.IDE.GHC.Compat.Util       as Util
 import           Development.IDE.GHC.ExactPrint
-import           Language.Haskell.GHC.ExactPrint.Transform (TransformT(TransformT))
-
-#if MIN_VERSION_ghc(9,4,1)
-
-import           GHC.Data.Bag (Bag)
-
-#endif
-
 import           GHC.Exts
-
-
-import           GHC.Parser.Annotation (SrcSpanAnn'(..))
-import qualified GHC.Types.Error as Error
-
-
+import qualified GHC.Types.Error                       as Error
+import           Ide.Plugin.Error                      (PluginError (PluginInternalError))
 import           Ide.Plugin.Splice.Types
 import           Ide.Types
-import           Language.Haskell.GHC.ExactPrint (uniqueSrcSpanT)
-import           Language.LSP.Server
-import           Language.LSP.Protocol.Types
+import qualified Language.LSP.Protocol.Lens            as J
 import           Language.LSP.Protocol.Message
-import qualified Language.LSP.Protocol.Lens         as J
-import Ide.Plugin.Error (PluginError(PluginInternalError))
+import           Language.LSP.Protocol.Types
+
+#if !MIN_VERSION_base(4,20,0)
+import           Data.Foldable                         (Foldable (foldl'))
+#endif
+
+#if MIN_VERSION_ghc(9,4,1)
+import           GHC.Data.Bag                          (Bag)
+#endif
+
+#if MIN_VERSION_ghc(9,9,0)
+import           GHC.Parser.Annotation                 (EpAnn (..))
+#else
+import           GHC.Parser.Annotation                 (SrcSpanAnn' (..))
+#endif
+
 
 descriptor :: PluginId -> PluginDescriptor IdeState
 descriptor plId =
@@ -95,10 +93,10 @@ expandTHSplice ::
     ExpandStyle ->
     CommandFunction IdeState ExpandSpliceParams
 expandTHSplice _eStyle ideState _ params@ExpandSpliceParams {..} = ExceptT $ do
-    clientCapabilities <- getClientCapabilities
+    clientCapabilities <- pluginGetClientCapabilities
     rio <- askRunInIO
     let reportEditor :: ReportEditor
-        reportEditor msgTy msgs = liftIO $ rio $ sendNotification SMethod_WindowShowMessage (ShowMessageParams msgTy (T.unlines msgs))
+        reportEditor msgTy msgs = liftIO $ rio $ pluginSendNotification SMethod_WindowShowMessage (ShowMessageParams msgTy (T.unlines msgs))
         expandManually :: NormalizedFilePath -> ExceptT PluginError IO WorkspaceEdit
         expandManually fp = do
             mresl <-
@@ -195,7 +193,7 @@ expandTHSplice _eStyle ideState _ params@ExpandSpliceParams {..} = ExceptT $ do
       Nothing -> pure $ Right $ InR Null
       Just (Left err) -> pure $ Left $ err
       Just (Right edit) -> do
-        _ <- sendRequest SMethod_WorkspaceApplyEdit (ApplyWorkspaceEditParams Nothing edit) (\_ -> pure ())
+        _ <- pluginSendRequest SMethod_WorkspaceApplyEdit (ApplyWorkspaceEditParams Nothing edit) (\_ -> pure ())
         pure $ Right $ InR Null
 
     where
@@ -207,7 +205,7 @@ setupHscEnv
     :: IdeState
     -> NormalizedFilePath
     -> ParsedModule
-    -> ExceptT PluginError IO (Annotated ParsedSource, HscEnv, DynFlags)
+    -> ExceptT PluginError IO (ParsedSource, HscEnv, DynFlags)
 setupHscEnv ideState fp pm = do
     hscEnvEq <- runActionE "expandTHSplice.fallback.ghcSessionDeps" ideState $
                     useE GhcSessionDeps fp
@@ -223,10 +221,10 @@ setupDynFlagsForGHCiLike env dflags = do
         platform = targetPlatform dflags3
         dflags3a = setWays hostFullWays dflags3
         dflags3b =
-            foldl gopt_set dflags3a $
+            foldl' gopt_set dflags3a $
                 concatMap (wayGeneralFlags platform) hostFullWays
         dflags3c =
-            foldl gopt_unset dflags3b $
+            foldl' gopt_unset dflags3b $
                 concatMap (wayUnsetGeneralFlags platform) hostFullWays
         dflags4 =
             dflags3c
@@ -245,7 +243,7 @@ adjustToRange uri ran (WorkspaceEdit mhult mlt x) =
             let minStart =
                     case L.fold (L.premap (view J.range) L.minimum) eds of
                         Nothing -> error "impossible"
-                        Just v -> v
+                        Just v  -> v
             in adjustLine minStart <$> eds
 
         adjustATextEdits :: Traversable f => f (TextEdit |? AnnotatedTextEdit) -> f (TextEdit |? AnnotatedTextEdit)
@@ -273,8 +271,13 @@ adjustToRange uri ran (WorkspaceEdit mhult mlt x) =
 -- `GenLocated`. In GHC >= 9.2 this will be a SrcSpanAnn', with annotations;
 -- earlier it will just be a plain `SrcSpan`.
 {-# COMPLETE AsSrcSpan #-}
+#if MIN_VERSION_ghc(9,9,0)
+pattern AsSrcSpan :: SrcSpan -> EpAnn ann
+pattern AsSrcSpan locA <- (getLoc -> locA)
+#else
 pattern AsSrcSpan :: SrcSpan -> SrcSpanAnn' a
 pattern AsSrcSpan locA <- SrcSpanAnn {locA}
+#endif
 
 findSubSpansDesc :: SrcSpan -> [(LHsExpr GhcTc, a)] -> [(SrcSpan, a)]
 findSubSpansDesc srcSpan =
@@ -305,7 +308,7 @@ instance HasSplice AnnListItem HsExpr where
 #if MIN_VERSION_ghc(9,5,0)
     type SpliceOf HsExpr = HsSpliceCompat
     matchSplice _ (HsUntypedSplice _ spl) = Just (UntypedSplice spl)
-    matchSplice _ (HsTypedSplice _ spl) = Just (TypedSplice spl)
+    matchSplice _ (HsTypedSplice _ spl)   = Just (TypedSplice spl)
 #else
     type SpliceOf HsExpr = HsSplice
     matchSplice _ (HsSpliceE _ spl) = Just spl
@@ -356,7 +359,7 @@ manualCalcEdit ::
     ClientCapabilities ->
     ReportEditor ->
     Range ->
-    Annotated ParsedSource ->
+    ParsedSource ->
     HscEnv ->
     TcGblEnv ->
     RealSrcSpan ->
@@ -394,7 +397,7 @@ manualCalcEdit clientCapabilities reportEditor ran ps hscEnv typechkd srcSpan _e
                                                             (fst <$> expandSplice astP spl)
                                                     )
                                         Just <$> case eExpr of
-                                            Left x -> pure $ L _spn x
+                                            Left x  -> pure $ L _spn x
                                             Right y -> unRenamedE dflags y
                                     _ -> pure Nothing
             let (warns, errs) =
@@ -471,7 +474,7 @@ fromSearchResult _        = Nothing
 -- TODO: Declaration Splices won't appear in HieAst; perhaps we must just use Parsed/Renamed ASTs?
 codeAction :: PluginMethodHandler IdeState Method_TextDocumentCodeAction
 codeAction state plId (CodeActionParams _ _ docId ran _) = do
-    verTxtDocId <- lift $ getVersionedTextDoc docId
+    verTxtDocId <- lift $ pluginGetVersionedTextDoc docId
     liftIO $ fmap (fromMaybe ( InL [])) $
         runMaybeT $ do
             fp <- MaybeT $ pure $ uriToNormalizedFilePath $ toNormalizedUri theUri
@@ -506,12 +509,12 @@ codeAction state plId (CodeActionParams _ _ docId ran _) = do
                         | spanIsRelevant l ->
                             case expr of
 #if MIN_VERSION_ghc(9,5,0)
-                                HsTypedSplice{} -> Here (spLoc, Expr)
+                                HsTypedSplice{}   -> Here (spLoc, Expr)
                                 HsUntypedSplice{} -> Here (spLoc, Expr)
 #else
-                                HsSpliceE {} -> Here (spLoc, Expr)
+                                HsSpliceE {}      -> Here (spLoc, Expr)
 #endif
-                                _            -> Continue
+                                _                 -> Continue
                     _ -> Stop
                 )
                 `extQ` \case
